import { omit, useWith, equals, applySpec, nth, always, o, map, toPairs, curry, isNil, prop, filter, complement } from 'ramda';
import { equalsLength } from 'ramda-extension';

var omitTemporalKeys = omit(['version']);
/**
 * Returns whether the entries are equal, ignoring the version.
 *
 * @param {Object} entry1 first entry
 * @param {Object} entry2 second entry
 * @returns {boolean} whether the entries are equal
 */

var areEntriesEqual = useWith(equals, [omitTemporalKeys, omitTemporalKeys]);

/**
 * Converts the input of `store.injectSomething()` or `store.ejectSomething()`
 * to an array of standalone entries.
 *
 * @param {Object} injectables an object with injectables as values
 * @param {?string} namespace namespace of all the entries
 * @param {?number} version version of all the entries
 * @returns {Object[]} an array of entries
 */

var createEntries = function createEntries(injectables, namespace, version) {
  var createEntry = applySpec({
    key: nth(0),
    value: nth(1),
    namespace: always(namespace),
    version: always(version)
  });
  return o(map(createEntry), toPairs)(injectables);
};

var getVersion = prop('version');
/**
 * Returns whether the version of a tested entry is not higher than the reference version.
 *
 * @param {?number} referenceVersion version to compare against
 * @param {Object} testedEntry entry to test
 * @returns {boolean} whether the version can be ejected
 */

var isEntryEjectableByVersion = curry(function (referenceVersion, testedEntry) {
  var testedVersion = getVersion(testedEntry);

  if (isNil(referenceVersion) && isNil(testedVersion)) {
    return true;
  }

  if (isNil(referenceVersion) || isNil(testedVersion)) {
    return false;
  }

  return testedVersion <= referenceVersion;
});

/**
 * Returns whether an entry is included exactly N times in an array of entries, ignoring version.
 *
 * @param {number} times how many times the entry must be included
 * @param {Object[]} entries entries to search through
 * @param {Object} entry entry to search for
 * @returns {boolean} whether the entry is included exactly N times
 */

var isEntryIncludedTimes = curry(function (times, entries, entry) {
  return o(equalsLength(times), filter(areEntriesEqual(entry)))(entries);
});

/**
 * Returns whether an entry is NOT included in an array of entries, ignoring version.
 *
 * @param {Object[]} entries entries to search through
 * @param {Object} entry entry to search for
 * @returns {boolean} whether the entry is NOT included
 */

var isEntryNotIncluded = isEntryIncludedTimes(0);

/**
 * Returns whether an entry is included in an array of entries, ignoring version.
 *
 * @param {Object[]} entries entries to search through
 * @param {Object} entry entry to search for
 * @returns {boolean} whether the entry is included
 */

var isEntryIncluded = complement(isEntryNotIncluded);

export { areEntriesEqual, createEntries, isEntryEjectableByVersion, isEntryIncluded, isEntryIncludedTimes, isEntryNotIncluded };
