'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var ramda = require('ramda');
var ramdaExtension = require('ramda-extension');

var omitTemporalKeys = ramda.omit(['version']);
/**
 * Returns whether the entries are equal, ignoring the version.
 *
 * @param {Object} entry1 first entry
 * @param {Object} entry2 second entry
 * @returns {boolean} whether the entries are equal
 */

var areEntriesEqual = ramda.useWith(ramda.equals, [omitTemporalKeys, omitTemporalKeys]);

/**
 * Converts the input of `store.injectSomething()` or `store.ejectSomething()`
 * to an array of standalone entries.
 *
 * @param {Object} injectables an object with injectables as values
 * @param {?string} namespace namespace of all the entries
 * @param {?number} version version of all the entries
 * @returns {Object[]} an array of entries
 */

var createEntries = function createEntries(injectables, namespace, version) {
  var createEntry = ramda.applySpec({
    key: ramda.nth(0),
    value: ramda.nth(1),
    namespace: ramda.always(namespace),
    version: ramda.always(version)
  });
  return ramda.o(ramda.map(createEntry), ramda.toPairs)(injectables);
};

var getVersion = ramda.prop('version');
/**
 * Returns whether the version of a tested entry is not higher than the reference version.
 *
 * @param {?number} referenceVersion version to compare against
 * @param {Object} testedEntry entry to test
 * @returns {boolean} whether the version can be ejected
 */

var isEntryEjectableByVersion = ramda.curry(function (referenceVersion, testedEntry) {
  var testedVersion = getVersion(testedEntry);

  if (ramda.isNil(referenceVersion) && ramda.isNil(testedVersion)) {
    return true;
  }

  if (ramda.isNil(referenceVersion) || ramda.isNil(testedVersion)) {
    return false;
  }

  return testedVersion <= referenceVersion;
});

/**
 * Returns whether an entry is included exactly N times in an array of entries, ignoring version.
 *
 * @param {number} times how many times the entry must be included
 * @param {Object[]} entries entries to search through
 * @param {Object} entry entry to search for
 * @returns {boolean} whether the entry is included exactly N times
 */

var isEntryIncludedTimes = ramda.curry(function (times, entries, entry) {
  return ramda.o(ramdaExtension.equalsLength(times), ramda.filter(areEntriesEqual(entry)))(entries);
});

/**
 * Returns whether an entry is NOT included in an array of entries, ignoring version.
 *
 * @param {Object[]} entries entries to search through
 * @param {Object} entry entry to search for
 * @returns {boolean} whether the entry is NOT included
 */

var isEntryNotIncluded = isEntryIncludedTimes(0);

/**
 * Returns whether an entry is included in an array of entries, ignoring version.
 *
 * @param {Object[]} entries entries to search through
 * @param {Object} entry entry to search for
 * @returns {boolean} whether the entry is included
 */

var isEntryIncluded = ramda.complement(isEntryNotIncluded);

exports.areEntriesEqual = areEntriesEqual;
exports.createEntries = createEntries;
exports.isEntryEjectableByVersion = isEntryEjectableByVersion;
exports.isEntryIncluded = isEntryIncluded;
exports.isEntryIncludedTimes = isEntryIncludedTimes;
exports.isEntryNotIncluded = isEntryNotIncluded;
